/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2013 University of Washington
 * Copyright (c) 2015 Bucknell University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Mitch Watrous (watrous@u.washington.edu)
 * Modified by: Li Li (ll024@bucknell.edu)
 */

#ifndef GNUPLOT_HELPER_H
#define GNUPLOT_HELPER_H

#include <map>
#include <utility>
#include <string>
#include "ns3/object-factory.h"
#include "ns3/ptr.h"
#include "ns3/deprecated.h"
#include "ns3/probe.h"
#include "ns3/gnuplot-aggregator.h"
#include "ns3/event-driven-collector.h"
#include "ns3/time-series-collector.h"
#include "ns3/nstime.h"
#include "ns3/scaling-collector.h"

namespace ns3 {

/**
 * \ingroup gnuplot
 * \brief Helper class used to make gnuplot plots.
 **/
class GnuplotHelper
{
public:
  /**
   * Constructs a gnuplot helper that will create a space separated
   * gnuplot data file named "gnuplot-helper.dat", a gnuplot control
   * file named "gnuplot-helper.plt", and a shell script to generate
   * the gnuplot named "gnuplot-helper.sh" unless it is later
   * configured otherwise.
   */
  GnuplotHelper ();

  /**
   * \param outputFileNameWithoutExtension name of gnuplot related files to
   * write with no extension
   * \param title plot title string to use for this plot.
   * \param xLegend the legend for the x horizontal axis.
   * \param yLegend the legend for the y vertical axis.
   * \param terminalType terminal type setting string for output. The
   * default terminal type is "png"
   *
   * Constructs a gnuplot helper that will create a space separated
   * gnuplot data file named outputFileNameWithoutExtension + ".dat",
   * a gnuplot control file named outputFileNameWithoutExtension +
   * ".plt", and a shell script to generate the gnuplot named
   * outputFileNameWithoutExtension + ".sh".
   */
  GnuplotHelper (const std::string &outputFileNameWithoutExtension,
                 const std::string &title,
                 const std::string &xLegend,
                 const std::string &yLegend,
                 const std::string &terminalType = "png") NS_DEPRECATED;

  virtual ~GnuplotHelper ();

  /**
   * \param outputFileNameWithoutExtension name of gnuplot related files to
   * write with no extension
   * \param title plot title string to use for this plot.
   * \param xLegend the legend for the x horizontal axis.
   * \param yLegend the legend for the y vertical axis.
   * \param terminalType terminal type setting string for output. The
   * default terminal type is "png"   
   *
   * Configures plot related parameters for this gnuplot helper so
   * that it will create a space separated gnuplot data file named
   * outputFileNameWithoutExtension + ".dat", a gnuplot control file
   * named outputFileNameWithoutExtension + ".plt", and a shell script
   * to generate the gnuplot named outputFileNameWithoutExtension +
   * ".sh".
   */
  void ConfigurePlot (const std::string &outputFileNameWithoutExtension,
                      const std::string &title,
                      const std::string &xLegend,
                      const std::string &yLegend,
                      const std::string &terminalType = "png");

  /**
   * \param terminalType terminal type setting string for output. 
   */
  void SetTerminalType (const std::string &terminalType);

  /**
   * Data output from the collector may be scaled down by this amount
   * in the resultant values; e.g. select a value of 1000 to make the 
   * y-axis units in terms of thousands.  Default value is 1.
   * Note that this function must be called before AddProbe() to have
   * effect on the next probe trace source.
   *
   * \param scalingFactor the scaling factor factor applied on output
   */
  void SetScalingFactor (double scalingFactor); 

  /**
   * Select the collector type and configure its attributes for any probes 
   * added thereafter (but not affecting probes added before).  
   * The default is "ns3::EventDrivenCollector".
   * \param collectorType the ns-3 TypeId string of the collector type
   * \param n0 the name of the attribute to set
   * \param v0 the value of the attribute to set
   * \param n1 the name of the attribute to set
   * \param v1 the value of the attribute to set
   * \param n2 the name of the attribute to set
   * \param v2 the value of the attribute to set
   * \param n3 the name of the attribute to set
   * \param v3 the value of the attribute to set
   * \param n4 the name of the attribute to set
   * \param v4 the value of the attribute to set
   * \param n5 the name of the attribute to set
   * \param v5 the value of the attribute to set
   * \param n6 the name of the attribute to set
   * \param v6 the value of the attribute to set
   * \param n7 the name of the attribute to set
   * \param v7 the value of the attribute to set
   */
  void SetCollectorType (std::string collectorType,
                         std::string n0 = "", const AttributeValue &v0 = EmptyAttributeValue (),
                         std::string n1 = "", const AttributeValue &v1 = EmptyAttributeValue (),
                         std::string n2 = "", const AttributeValue &v2 = EmptyAttributeValue (),
                         std::string n3 = "", const AttributeValue &v3 = EmptyAttributeValue (),
                         std::string n4 = "", const AttributeValue &v4 = EmptyAttributeValue (),
                         std::string n5 = "", const AttributeValue &v5 = EmptyAttributeValue (),
                         std::string n6 = "", const AttributeValue &v6 = EmptyAttributeValue (),
                         std::string n7 = "", const AttributeValue &v7 = EmptyAttributeValue ());

  /**
   * Set the key location on the plot; the default is KEY_INSIDE.
   * \param keyLocation the location of the key in the plot.
   */
  void SetKeyLocation (enum GnuplotAggregator::KeyLocation keyLocation);


  /**
   * \param typeId the type ID for the probe used when it is created.
   * \param path Config path for underlying trace source to be probed
   * \param probeTraceSource the probe trace source to access.
   * \param title the title to be associated to this dataset
   * \param keyLocation the location of the key in the plot.
   *
   * Plots a dataset generated by hooking the ns-3 trace source with a
   * probe, and then plot the values from the probeTraceSource. The dataset
   * will have the provided title, and will consist of the 'newValue'
   * at each timestamp.
   *
   * This method will create one or more probes according to the TypeId 
   * provided, connect the probe(s) to the trace source specified by
   * the config path, and hook the probeTraceSource(s) to the downstream 
   * aggregator.
   *
   * If the config path has more than one match in the system
   * (e.g. there is a wildcard), then one dataset for each match will
   * be plotted.  The dataset titles will be suffixed with the matched
   * characters for each of the wildcards in the config path,
   * separated by spaces.  For example, if the proposed dataset title
   * is the string "bytes", and there are two wildcards in the path,
   * then dataset titles like "bytes-0 0" or "bytes-12 9" will be
   * possible as labels for the datasets that are plotted.
   */
  void PlotProbe (const std::string &typeId,
                  const std::string &path,
                  const std::string &probeTraceSource,
                  const std::string &title,
                  enum GnuplotAggregator::KeyLocation keyLocation = GnuplotAggregator::KEY_INSIDE) NS_DEPRECATED;

  /**
   * \param typeId the type ID for the probe used when it is created.
   * \param path Config path for underlying trace source to be probed
   * \param probeTraceSource the probe trace source to access.
   * \param dataSeriesName the data series name to be associated to this dataset
   *
   * Plots one or more datasets on the previously configured plot by hooking 
   * the ns-3 trace source with a probe, and then plotting the values from the 
   * probeTraceSource. The dataset
   * will have the provided title, and will consist of the 'newValue'
   * at each timestamp.  The currently configured collector type will
   * determine the policy for exporting data to the GnuplotAggregator.
   *
   * If the config path has more than one match in the system
   * (e.g. there is a wildcard), then one dataset for each match will
   * be plotted.  The dataset titles will be suffixed with the matched
   * characters for each of the wildcards in the config path,
   * separated by spaces.  For example, if the proposed dataset title
   * is the string "bytes", and there are two wildcards in the path,
   * then dataset titles like "bytes-0 0" or "bytes-12 9" will be
   * possible as labels for the datasets that are plotted.
   */
  void AddProbe (const std::string &typeId,
                 const std::string &path,
                 const std::string &probeTraceSource,
                 const std::string &dataSeriesName);

  /**
   * \param probeName the probe's name.
   * \return Ptr to probe
   * \brief Gets the specified probe.
   */
  Ptr<Probe> GetProbe (std::string probeName) const NS_DEPRECATED;

  /**
   * \return Ptr to GnuplotAggregator object
   * \brief Gets the aggregator.
   *
   * This function is non-const because an aggregator may be lazily
   * created by this method.
   */
  Ptr<GnuplotAggregator> GetAggregator () NS_DEPRECATED;

private:

  /**
   * \param typeId the type ID for the probe used when it is created.
   * \param probeName the probe's name.
   * \param path Config path to access the probe.
   *
   * \brief Adds a probe to be used to make the plot.
   */
  void AddProbeToMap (const std::string &typeId,
                 const std::string &probeName,
                 const std::string &path);

  /**
   * \brief Constructs the aggregator.
   */
  void ConstructAggregator ();

  /**
   * \param typeId the type ID for the probe used when it is created.
   * \param matchIdentifier this string is used to make the probe's
   * context be unique.
   * \param path Config path to access the probe.
   * \param probeTraceSource the probe trace source to access.
   * \param title the title to be associated to this dataset.
   *
   * \brief Creates the data collection data flow chain, including
   * a probe, one or more collectors, and an aggregator
   */
  void CreateDataCollectionChain (const std::string &typeId,
                                 const std::string &matchIdentifier,
                                 const std::string &path,
                                 const std::string &probeTraceSource,
                                 const std::string &title);

  /**
   * \param collectorName the name of the collector
   *
   * \brief Adds the collector to the GnuplotHelper.
   */
  void AddCollector(const std::string &collectorName);

  /**
   * \param collectorName the name of the ScalingCollector
   *
   * \brief Adds a scaling collector
   */ 
  void AddScalingCollector(const std::string &collectorName);

  /**
   * \param probeTraceSource the trace source of the probe
   * \param probeName the name of the probe in the map
   * \param probeContext the context of the probe in the collector map
   *
   * \brief Connects a probe to a Collector object
   */
  void ConnectProbeToCollector(const std::string probeTraceSource,
                               const std::string probeName, 
                               const std::string probeContext);

  /**
   * \param probeContext the name of the collector in the map
   *
   * \brief connect a collector to an aggregator
   */
  void ConnectCollectorToAggregator(const std::string probeContext);


  /// Used to create the probes and collectors as they are added.
  ObjectFactory m_factory;

  /// The aggregator used to make the plots.
  Ptr<GnuplotAggregator> m_aggregator;

  /// Maps probe names to probes.
  std::map<std::string, std::pair <Ptr<Probe>, std::string> > m_probeMap;

  /// Maps Collector names to Collectors.
  std::map<std::string, Ptr<Collector> > m_collectorMap;

  // Maps ScalingCollector names to ScalingCollector
  std::map<std::string, Ptr<ScalingCollector> > m_scalingCollectorMap;

  /// Number of plot probes that have been created.
  uint32_t m_plotProbeCount;

  /// The name of the output file to created without its extension.
  std::string m_outputFileNameWithoutExtension;

  /// Title string to use for this plot.
  std::string m_title;

  /// Legend for the x axis.
  std::string m_xLegend;

  /// Legend for the y axis.
  std::string m_yLegend;

  /// Terminal type for the plot.
  std::string m_terminalType;

  /// Factory object for instantiating collector
  ObjectFactory m_collectorFactory;
  
  /// The scaling factor 
  double m_scalingFactor;

  /// The key location
  enum GnuplotAggregator::KeyLocation m_keyLocation;

}; // class GnuplotHelper


} // namespace ns3

#endif // GNUPLOT_HELPER_H
