/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2011 Bucknell University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors: Tiago G. Rodrigues (tgr002@bucknell.edu)
 */

#ifndef DATA_COLLECTION_OBJECT_H
#define DATA_COLLECTION_OBJECT_H

#include <string>
#include "ns3/object.h"
#include "ns3/nstime.h"

namespace ns3 {

/**
 * \ingroup aggregator
 *
 * Base class for data collection framework objects.
 *
 * All data collection objects have 1) a string name, and 2) enabled
 * or disabled status.
 */
class DataCollectionObject : public Object
{
public:
  /**
   * \brief Get the type ID.
   * \return the object TypeId
   */
  static TypeId GetTypeId ();

  DataCollectionObject ();
  virtual ~DataCollectionObject ();

  /// Set the status of an individual object.
  virtual void Enable (void);
  /// Unset the status of an individual object.
  virtual void Disable (void);

  /// Check the status of an individual object.
  virtual bool IsEnabled (void) const;

  /// Get the object's name.
  std::string GetName (void) const;

  /// Set the object's name.  All spaces are replaced by underscores.
  void SetName (std::string name);

  /// Define the Collector callback function signature. This is used
  /// in the AddTraceSource() function in Collector class.
  typedef void (* CollectorTraceCallback) (const double timeStamp, const bool value);

  /**
   * \param startTime the time the DataCollectionObject is enabled
   * \param endTime the time the DataCollectionObject is disabled
   *
   * \brief set the time period in which the collector will be enabled.
   *
   * Note that once this method is called, you do not need to explicitly call
   * Collector::Enable() or Collector::Disable() again. The Collector class
   * will take care of everything for you.
   *
   * A good pratice will be that after calling SetEnabledPeriod(), do not
   * mess around with Enable() and Disable() anymore. This function takes
   * care of everything for you.
   */
  void SetEnabledPeriod (Time startTime, Time endTime);

  /**
   * \brief Disable the enabled period so that you can control the collector
   * manually with Enable() and Disable()
   */
  void UnsetEnabledPeriod (void);

  /**
   * \brief Get the starting time of the DataCollectionObject
   *
   * \return the starting time of the DataCollectionObject
   */
  Time GetStartTime (void)
  {
    return m_startTime;
  }

  /**
   * \brief Get the ending time of the DataCollectionObject
   *
   * \return the ending time of the DataCollectionObject
   */
  Time GetEndTime (void)
  {
    return m_endTime;
  }


  /**
   * Return whether or not the enabled period is set
   *
   * \return a boolean value indicating whether or not the enabled period is set
   */
  bool IsEnabledPeriodSet (void)
  {
    return isEnabledPeriodSet;
  }


protected:
  /// Object's activation state.
  bool m_enabled;

  /// Name of the object within the data collection framework
  std::string m_name;

  /// Report Data to downstream objects
  void ReportData (void);

  /// The starting time of the DataCollectionObject
  Time m_startTime;

  /// The ending time of the DataCollectionObject
  Time m_endTime;

  /// Whether or not the enabled period is set
  bool isEnabledPeriodSet;
};

} // namespace ns3

#endif // DATA_COLLECTION_OBJECT_H
