/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2015 Bucknell University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Li Li (ll024@bucknell.edu)
 */

#include <iostream>
#include <string>
#include "ns3/time-averaging-collector.h"
#include "ns3/double-probe.h"
#include "ns3/test.h"
#include "ns3/core-module.h"

using namespace ns3;

class TimeAveragingCollectorSampleEmitter : public Object
{
public:
  static TypeId GetTypeId (void);
  TimeAveragingCollectorSampleEmitter ()
  {
    m_var = CreateObject<ExponentialRandomVariable> ();
  }
  virtual ~TimeAveragingCollectorSampleEmitter ()
  {
  }
  void Start ()
  {
    Reschedule ();
  }
  void Reschedule ()
  {
    m_time = m_var->GetValue ();
    Simulator::Schedule (Seconds (m_time), &TimeAveragingCollectorSampleEmitter::Report, this);
    m_time += Simulator::Now ().GetSeconds ();
  }
  double GetTime ()
  {
    return m_time;
  }
  double GetValue ()
  {
    return aux;
  }
private:
  void Report ()
  {
    aux = m_var->GetValue ();
    m_trace = aux;
    Reschedule ();
  }
  Ptr<ExponentialRandomVariable> m_var;
  double m_time;
  TracedValue<double> m_trace;
  double aux;
};


TypeId
TimeAveragingCollectorSampleEmitter::GetTypeId (void)
{
  static TypeId tid = TypeId ("TimeAveragingCollectorSampleEmitter")
    .SetParent<Object> ()
    .AddTraceSource ("Emitter", "XX", MakeTraceSourceAccessor (&TimeAveragingCollectorSampleEmitter::m_trace), "ns3::TracedValue::DoubleCallback")
  ;
  return tid;
}


// ===========================================================================
// Test case
// ===========================================================================

class TimeAveragingCollectorTestCase1 : public TestCase
{
public:
  TimeAveragingCollectorTestCase1 ();
  virtual ~TimeAveragingCollectorTestCase1 ();

private:
  double currentTime;
  double average;
  double batchCount;
  virtual void DoRun (void);
  void TraceSinkCallback (std::string context, double oldValue, double newValue);
  void ProbeTraceSinkCallback (double time, double newValue);
  Ptr<TimeAveragingCollector> collector;
  Ptr<TimeAveragingCollectorSampleEmitter> s;
};

TimeAveragingCollectorTestCase1::TimeAveragingCollectorTestCase1 ()
  : TestCase ("TimeAveragingCollector test case 1")
{
}

TimeAveragingCollectorTestCase1::~TimeAveragingCollectorTestCase1 ()
{
}

void
TimeAveragingCollectorTestCase1::DoRun (void)
{
  // let the collector generate outputs every 1 second
  collector = CreateObject<TimeAveragingCollector> ();
  collector->SetPeriod (Seconds (3));
  collector->Enable ();
  s = CreateObject<TimeAveragingCollectorSampleEmitter> ();
  currentTime = 3;
  average = 0;
  batchCount = 0;

  Ptr<DoubleProbe> p = CreateObject<DoubleProbe> ();
  p->SetName ("TimeAveragingCollectorSampleProbe");

  Simulator::Schedule (Seconds (0), &TimeAveragingCollectorSampleEmitter::Start, s);
  p->SetAttribute ("Start", TimeValue (Seconds (0.0)));
  p->SetAttribute ("Stop", TimeValue (Seconds (100.0)));
  Simulator::Stop (Seconds (200));

  Names::Add ("/Names/TimeAveragingCollectorSampleEmitter", s);

  // Hook probe to the emitter.
  p->ConnectByObject ("Emitter", s);

  // Hook the collector to the probe
  bool connected = p->TraceConnectWithoutContext ("Output",  MakeCallback (&TimeAveragingCollector::TraceSinkDouble, collector));
  NS_ASSERT (connected == true);

  // Connect the probe to the test trace sink defined in this class
  connected = p->TraceConnectWithoutContext ("Output",  MakeCallback (&TimeAveragingCollectorTestCase1::ProbeTraceSinkCallback, this));
  NS_ASSERT (connected == true);

  // connect the collector to the TraceSinkCallback.
  connected = collector->TraceConnect ("Output", p->GetName (), MakeCallback (&TimeAveragingCollectorTestCase1::TraceSinkCallback, this));
  NS_ASSERT (connected == true);
  NS_UNUSED (connected);

  Simulator::Run ();
  Simulator::Destroy ();
}

void
TimeAveragingCollectorTestCase1::TraceSinkCallback (std::string context, double time, double newValue)
{
  // check whether the collector is generating outputs every 3 second.
  NS_TEST_ASSERT_MSG_EQ_TOL (currentTime, time, 0.0001, "Time generated by the collector different than the actual time");

  // check whether the average value generated by the collector is correct
  NS_TEST_ASSERT_MSG_EQ_TOL (average, newValue, 0.0001, "Average generated by the collector different than the actual average");
  batchCount = 0;
  currentTime += 3;
}

void
TimeAveragingCollectorTestCase1::ProbeTraceSinkCallback (double time, double newValue)
{
  average = (average * batchCount + newValue) / (batchCount + 1);
  batchCount++;
}

class TimeAveragingCollectorTestSuite : public TestSuite
{
public:
  TimeAveragingCollectorTestSuite ();
};

TimeAveragingCollectorTestSuite::TimeAveragingCollectorTestSuite ()
  : TestSuite ("time-averaging-collector", UNIT)
{
  AddTestCase (new TimeAveragingCollectorTestCase1, TestCase::QUICK);
}

static TimeAveragingCollectorTestSuite TimeAveragingCollectorTestSuite;

